//-----------------------------------------------------
// Nine linked rings
//
// Windows console program 
// compiled by Orwell Dev-C++ v5.2.0.3
// http://sourceforge.net/projects/orwelldevcpp/
//
// 01/15/2013 by Simon Hung
//-----------------------------------------------------

#include <stdio.h>
#include <windows.h>

#define RING_DOWN (0)
#define RING_UP   (1)
#define MAX_ITEMS (12)

void allRingUp(int n);
void allRingDown(int n);
void printRing(int n, int direct);

void drawRing(int n, int direct);


static const char *directStr[] = { "Down", "Up  " };
static int stepCounter;

//------------------------------------------------------------
// allRingDown:
//------------------------------------------------------------               
// (1) - - - - - ==> _ _ _ - - .... allRingDown(n-2)

// (2) _ _ _ - - ==> _ _ _ - _ .... printRingDown(n)

// (3) _ _ _ - _ ==> - - - - _ .... allRingUp(n-2)

// (4) - - - - _ ==> _ _ _ _ _ .... allRingDown(n-1)
//------------------------------------------------------------

void allRingDown(int n)
{
     if (n > 0) {
           allRingDown(n-2);
           printRing(n, RING_DOWN);
           allRingUp(n-2);
           allRingDown(n-1);
     }
}

//------------------------------------------------------------
// allRingUp:
//------------------------------------------------------------ 
// (1) _ _ _ _ _ ==> - - - - _ .... allRingUp(n-1)

// (2) - - - - _ ==> _ _ _ - _ .... allRingDown(n-2)

// (3) _ _ _ - _ ==> _ _ _ - - .... printRingUp(n)

// (4) _ _ _ - - ==> - - - - - .... allRingUp(n-2)
//------------------------------------------------------------

void allRingUp(int n)
{
     if (n > 0) {
           allRingUp(n-1);
           allRingDown(n-2);
           printRing(n, RING_UP);
           allRingUp(n-2);
     }
}


void printRing(int n, int direct)
{
     system("CLS"); //windows console command
     printf("%03d ------- %d %s -------\n\n", ++stepCounter, n, directStr[direct]);
     drawRing(n, direct);
     Sleep(500);
}


////////////////////////////////////////////////////////////////////
// BEGIN for draw ring status //////////////////////////////////////
////////////////////////////////////////////////////////////////////

static int flagArray[MAX_ITEMS];
static int totalItems;

//moveRing (0..n-1)
void drawRing(int n, int direct)
{
     int i;
     int moveRing = n-1; //(0 .. n-1) for match with flagArray
     
     flagArray[moveRing] = direct;
     
     //draw upper ring 
     for (i = 0; i < totalItems; i++) {
         if(flagArray[i] == RING_UP) {
             printf("O");            
         } else {
             if( moveRing == i) {
                 printf("v");
             } else {
                 printf(" ");   
             }
                          
         }
     }
     printf("\n");
     
     //draw lower ring
     for (i = 0; i < totalItems; i++) {
         if(flagArray[i] == RING_DOWN) {
             printf("O");            
         } else {
             if( moveRing == i) {
                 printf("^");
             } else {
                 printf(" ");   
             }
                          
         }
     }
     printf("\n\n");
}

void startAllRingUp(int n)
{
     int i;
    
     if(n > MAX_ITEMS) {
          printf("Too many items\n");
          return;
     }
     stepCounter = 0;
     totalItems = n;
     //initial all items down
     for(i = 0 ; i < MAX_ITEMS; i++) 
         flagArray[i] =  RING_DOWN; 
         
     allRingUp(n);    
}

void startAllRingDown(int n)
{
     int i;
     
     if(n > MAX_ITEMS) {
          printf("Too many items\n");
          return;
     }

     stepCounter = 0;
     totalItems = n;
     //initial all items up
     for(i = 0 ; i < MAX_ITEMS; i++) 
         flagArray[i] =  RING_UP; 
         
     allRingDown(n);    
}
//////////////////////////////////////////////////////////////////
// END for draw ring status //////////////////////////////////////
//////////////////////////////////////////////////////////////////

int main(void)
{
     //startAllRingUp(9);
     startAllRingDown(7);
     system("pause"); //windows console command
     return 0;
}
